// These variables are set by other scripts (e.g. generated by the docs task).

export function getSidebarNodes () {
  return window.sidebarNodes || {}
}

export function getVersionNodes () {
  return window.versionNodes || []
}
