defmodule Common.Nesting.Prefix.B.A do
  @moduledoc "moduledoc"
end

defmodule Common.Nesting.Prefix.B.B.A do
  @moduledoc "moduledoc"
end

defmodule Common.Nesting.Prefix.B.C do
  @moduledoc "moduledoc"
end

defmodule Common.Nesting.Prefix.C do
  @moduledoc "moduledoc"
end
