#quick-switch-modal-body {
  width: 100%;
  position: relative;
}

#quick-switch-modal-body .ri-search-2-line {
  position: absolute;
  left: 0;
  top: 0;
  padding: 4px 10px;
  color: var(--quickSwitchContour);
  font-weight: bold;
}

#quick-switch-modal-body #quick-switch-input {
  width: 100%;
  padding: 8px 6px 8px 38px;
  border: none;
  color: var(--quickSwitchInput);
  background-color: transparent;
  border-bottom: 1px solid var(--quickSwitchContour);
  box-sizing: border-box;
  transition: all .12s ease-out;
}

#quick-switch-modal-body #quick-switch-results {
  margin: 0;
}

#quick-switch-modal-body .quick-switch-result {
  padding: 2px 5px;
  border-bottom: 1px dotted var(--quickSwitchContour);
  transition: all .12s ease-out;
}

#quick-switch-modal-body .quick-switch-result:last-child {
  border-bottom: none;
}

#quick-switch-modal-body .quick-switch-result:hover {
  cursor: pointer;
}

#quick-switch-modal-body .quick-switch-result:is(:hover, .selected) {
  border-left: 4px solid var(--main);
  background-color: var(--codeBackground);
}
