.content-inner {
  font-family: var(--defaultFontFamily);
  font-size: 1em;
  line-height: 1.6875em;
  position: relative;
  background-color: var(--background);
  color: var(--textBody);
}

.content-inner :is(h1, h2, h3, h4, h5, h6) {
  font-family: var(--sansFontFamily);
  font-weight: 700;
  line-height: 1.5em;
  word-wrap: break-word;
  color: var(--textHeaders);
}

.content-inner h1 {
  font-size: 2em;
  margin: 0.5em 0;
}

.content-inner h1.signature {
  margin: 0;
}

.content-inner h1.section-heading {
  margin: 1.5em 0 0.5em;
}

.content-inner h1 small {
  font-weight: 300;
}

.content-inner h1 .icon-action {
  font-size: 1.2rem;
  font-weight: normal;
}

.content-inner h2 {
  font-size: 1.6em;
  margin: 1em 0 0.5em;
  font-weight: 700;
}

.content-inner h3 {
  font-size: 1.375em;
  margin: 1em 0 0.5em;
  font-weight: 700;
}

.content-inner li + li {
  margin-top: 0.25em;
}

.content-inner :is(a, .a-main) {
  color: var(--links);
  text-decoration: underline;
  text-decoration-color: var(--linksDecoration);
  text-decoration-skip-ink: auto;
}
.content-inner :is(a:visited, .a-main:visited) {
  color: var(--linksVisited);
}

.content-inner .icon-action {
  float: right;
  color: var(--iconAction);
  text-decoration: none;
  border: none;
  transition: color 0.3s ease-in-out;
  background-color: transparent;
  cursor: pointer;
}
.content-inner button.icon-action {
  margin-top: 12px;
}
.content-inner .icon-action:hover {
  color: var(--iconActionHover);
}
.content-inner .icon-action:visited {
  color: var(--iconAction);
}

.content-inner .livebook-badge-container {
  display: flex;
}

.content-inner a.livebook-badge {
  display: inline-flex;
}

.content-inner .note {
  color: var(--iconAction);
  margin-right: 5px;
  font-size: 14px;
  font-weight: normal;
}

.content-inner blockquote, .content-inner section.admonition {
  border-left: 3px solid var(--blockquoteBorder);
  position: relative;
  margin: 1.5625em 0;
  padding: 0 1.2rem;
  overflow: auto;
  background-color: var(--blockquoteBackground);
  border-radius: var(--borderRadius);
}
.content-inner blockquote p:last-child, .content-inner section.admonition p:last-child  {
  padding-bottom: 1em;
  margin-bottom: 0;
}

.content-inner table {
  margin: 2em 0;
  border-collapse: collapse;
}

.content-inner th {
  text-align: left;
  font-family: var(--sansFontFamily);
  text-transform: uppercase;
  font-weight: 700;
  padding-bottom: 0.5em;
}

.content-inner thead tr {
  border-bottom: 1px solid var(--tableHeadBorder);
}

.content-inner tbody tr {
  border-bottom: 1px solid var(--tableBodyBorder);
}

.content-inner tbody tr:last-child {
  border-bottom: none;
}

.content-inner tr {
  vertical-align: bottom;
  height: 2.5em;
}

.content-inner :is(td, th) {
  padding: 0.25em;
  padding-left: 1em;
  line-height: 2em;
  vertical-align: top;
}

.content-inner .section-heading {
  --icon-size: 16px;
  --icon-spacing: 5px;
  display: grid;
  grid-template: 1fr / 1fr;
}

.content-inner .section-heading > :is(.hover-link, .text) {
  grid-row: 1;
  grid-column: 1;
}

.content-inner .section-heading .hover-link {
  text-decoration: none;
}

.content-inner .section-heading i {
  font-size: var(--icon-size);
  margin-top: 0.1em;
  margin-left: calc(-1 * (var(--icon-size) + var(--icon-spacing)));
  padding-right: var(--icon-spacing); /* Avoids gap in hover area */
  opacity: 0;
}

@media screen and (max-width: 768px) {
  .content-inner .section-heading i {
    margin-left: calc(-1 * (var(--icon-size)));
  }
}

.content-inner :is(blockquote, section.admonition) .section-heading i {
  display: none;
}

.content-inner .section-heading .hover-link:is(:hover, :focus) i {
  opacity: 1;
}

/* Allow section link to be hovered and used “through” text */
.content-inner .section-heading .text {
  pointer-events: none;
}
.content-inner .section-heading .text a {
  pointer-events: all;
}

.content-inner .app-vsn {
  display: none !important;
  font-size: 0.6em;
  line-height: 1.5em;
}

@media screen and (max-width: 768px) {
  .content-inner .app-vsn {
    display: block !important;
  }
}

.content-inner img {
  max-width: 100%;
}

.content-inner strong > code {
  font-weight: bold;
}

.content-inner code {
  font-family: var(--monoFontFamily);
  font-style: normal;
  line-height: 24px;
  font-weight: 400;
}

@media screen and (max-width: 768px) {
  .content-inner :is(ol, ul) {
    padding-left: calc(1.5 * var(--content-gutter));
  }
}
