.content-inner .bottom-actions {
  display: flex;
  justify-content: space-between;
  margin-top: 4em;
}

.content-inner .bottom-actions .bottom-actions-button {
  display: flex;
  text-decoration: none;
  flex-direction: column;
  border-radius: var(--borderRadius);
  border: 1px solid var(--bottomActionsBtnBorder);
  padding: 8px 16px;
  min-width: 150px;
}

.content-inner .bottom-actions .bottom-actions-button .subheader {
  font-size: .8em;
  color: var(--bottomActionsBtnSubheader);
  white-space: nowrap;
}

.content-inner .bottom-actions .bottom-actions-button[rel="prev"] .subheader {
  text-align: right;
}

@media screen and (max-width: 768px) {
  .content-inner .bottom-actions {
    flex-direction: column-reverse;
  }

  .content-inner .bottom-actions .bottom-actions-item:not(:first-child) {
    margin-bottom: 16px;
  }
}
