.sidebar {
  --sidebarFontSize: 16px;
  --sidebarLineHeight: 20px;
  font-family: var(--sansFontFamily);
  font-size: var(--sidebarFontSize);
  font-weight: 300;
  line-height: var(--sidebarLineHeight);
  background-color: var(--sidebarBackground);
  color: var(--sidebarAccentMain);
  overflow: hidden;
  scrollbar-color: var(--sidebarScrollbarThumb) var(--sidebarScrollbarTrack);
}

.non-apple-os .sidebar {
  font-weight: 400; /* Non-Apple OSes render small light type too thinly */
}

.sidebar ul {
  list-style: none;
}

.sidebar ul li {
  margin: 0;
  padding: 0 10px;
}

.sidebar a {
  color: var(--sidebarAccentMain);
  text-decoration: none;
  transition: color 0.3s ease-in-out;
}

.sidebar a:hover {
  color: var(--sidebarHover);
}

.sidebar .sidebar-header {
  background-color: var(--sidebarHeader);
  width: 100%;
}

.sidebar .sidebar-projectInfo {
  display: flex;
  justify-content: start;
  align-items: center;
  gap: 16px;
  margin: 12px 16px 12px 14px;
  max-width: 235px;
}

.sidebar .sidebar-projectImage {
  align-self: flex-end;
}

.sidebar .sidebar-projectImage img {
  display: block;
  max-width: 48px;
  max-height: 48px;
}

.sidebar .sidebar-projectName {
  font-weight: 700;
  font-size: 20px;
  line-height: 24px;
  color: var(--sidebarAccentMain);
  margin: 0;
  padding: 0;
  max-width: 230px;
  word-wrap: break-word;
}

.sidebar .sidebar-projectVersion {
  display: block;
  position: relative;
  margin: 0;
  padding: 0;
  font-size: var(--sidebarFontSize);
  line-height: var(--sidebarLineHeight);
  color: var(--sidebarMuted);
}

.sidebar .sidebar-projectVersionsDropdown {
  cursor: pointer;
  position: relative;
  margin: 0;
  padding: 0 0 0 12px;
  border: none;
  -webkit-appearance: none;
  appearance: none;
  background-color: transparent;
  color: var(--sidebarMuted);
  z-index: 2;
}

.sidebar .sidebar-projectVersionsDropdown option {
  color: initial;
}

.sidebar .sidebar-projectVersionsDropdownCaret {
  position: absolute;
  left: 0;
  top: 2px;
  z-index: 1;
  font-size: 8px;
  color: var(--sidebarMuted);
}

.sidebar .sidebar-projectVersionsDropdown::-ms-expand {
  display: none;
}

.sidebar .sidebar-listNav {
  display: flex;
  margin: 0;
  padding: 4px 4px 0 4px;
}

.sidebar .sidebar-listNav :is(li, li button) {
  text-transform: uppercase;
  letter-spacing: 0.02em;
  font-size: 14px;
  color: var(--sidebarMuted);
}

.sidebar .sidebar-listNav li {
  display: inline-block;
  padding: 0;
}

.sidebar .sidebar-listNav li button {
  background: none;
  border: 0;
  border-radius: 0;
  -webkit-appearance: none;
  text-align: inherit;
  color: inherit;
  font-weight: inherit;
  cursor: pointer;
  display: inline-block;
  line-height: 27px;
  padding: 4px 10px 2px 10px;
  transition: all 150ms;
}

.sidebar .sidebar-listNav li:is(.selected) button {
  background-color: var(--sidebarBackground);
  border-top: var(--navTabBorderWidth) solid var(--sidebarLanguageAccentBar);
}

.sidebar .sidebar-listNav li:not(.selected) button {
  border-top: var(--navTabBorderWidth) solid var(--sidebarHeader);
}

.sidebar .sidebar-listNav li:is(:hover):not(.selected) button {
  background-color: var(--sidebarInactiveItemMarker);
  border-top: var(--navTabBorderWidth) solid var(--sidebarInactiveItemBorder);
  color: var(--sidebarAccentMain);
  transition: all 150ms;
}

.sidebar .sidebar-tabpanel {
  flex: 1 1 0.01%;
  overflow-y: auto;
  overscroll-behavior: contain;
  position: relative;
  -webkit-overflow-scrolling: touch;
  margin-top: 12px;
}

.sidebar .full-list {
  margin: 0;
  padding: 0 0 20px 0;
  position: relative;
}

.sidebar .full-list :is(li, a) {
  overflow: hidden;
  text-overflow: ellipsis;
}

.sidebar .full-list li {
  padding: 0;
  margin-right: 30px;
  line-height: 27px;
  white-space: nowrap;
}

.sidebar .full-list li.docs {
  margin-right: 0;
}

.sidebar .full-list li.open > ul {
  display: block;
  margin-left: 10px;
}

.sidebar .full-list li a.expand + button.icon-expand {
  appearance: none;
  background-color: transparent;
  border: 0;
  padding: 0;
  cursor: pointer;
  color: inherit;
  margin-right: 10px;
  font-size: calc(1.2 * var(--sidebarFontSize));
  line-height: var(--sidebarLineHeight);
  position: absolute;
  display: flex;
  right: 0;
  transform: translateY(calc(-100% - 4px));
}

.sidebar .full-list li a + button.icon-expand:after {
  font-family: remixicon;
  font-style: normal;
  -webkit-font-smoothing: antialiased;
  -moz-osx-font-smoothing: grayscale;
}

.sidebar .full-list li a.expand + button.icon-expand:after {
  content: var(--icon-arrow-down-s);
}

.sidebar .full-list li.open > a.expand + button.icon-expand:after {
  content: var(--icon-arrow-up-s);
}

.sidebar .full-list li.docs > a + button.icon-expand {
  margin-right: 12px;
  font-size: var(--sidebarFontSize);
  line-height: var(--sidebarFontSize);
  transform: translateY(calc(-100% - 5px));
}

.sidebar .full-list li.docs > a + button.icon-expand:after {
  content: var(--icon-add);
}

.sidebar .full-list li.docs.open > a + button.icon-expand:after {
  content: var(--icon-subtract);
}

.sidebar .full-list li.nesting-context {
  font-weight: bold;
  font-size: 0.9em;
  line-height: 1.8em;
  color: var(--sidebarSubheadings);
  margin-top: 10px;
  padding-left: 15px;
}

.sidebar .full-list li.group {
  text-transform: uppercase;
  font-weight: bold;
  font-size: 0.8em;
  margin: 1.5em 0 0;
  line-height: 1.8em;
  color: var(--sidebarSubheadings);
  padding-left: 15px;
}

.sidebar .full-list li a {
  padding: 3px 0 3px 15px;
  color: var(--sidebarItem);
}

.sidebar .full-list > li > a {
  display: block;
  width: 100%;
  height: 27px;
  line-height: var(--sidebarLineHeight);
}

.sidebar .full-list li .current-section > a {
  color: var(--sidebarActiveItem);
}

.sidebar .full-list li .current-section > a + button.icon-expand {
  color: var(--sidebarActiveItem);
}

.sidebar .full-list > li > a:hover {
  border-left: 3px solid var(--sidebarLanguageAccentBar);
  padding-left: 12px;
}

.sidebar .full-list > li.current-page > a {
  color: var(--sidebarActiveItem);
  border-left: 3px solid var(--sidebarLanguageAccentBar);
  padding-left: 12px;
}

.sidebar .full-list > li.current-page > a:after,
.sidebar .full-list > li.current-page {
  color: var(--sidebarActiveItem);
}

.sidebar .full-list > li:last-child {
  margin-bottom: 30px;
}

.sidebar .full-list > li.group:first-child {
  margin-top: 0;
}

.sidebar .full-list ul {
  display: none;
  margin: 10px 15px;
  margin-right: 0;
  padding: 0;
}

.sidebar .full-list ul li {
  font-weight: 300;
  line-height: var(--sidebarFontSize);
  padding: 0 8px;
  margin-right: 0;
  color: var(--sidebarAccentMain);
}

.non-apple-os .sidebar .full-list ul li {
  font-weight: 400; /* Non-Apple OSes render small light type too thinly */
}

.sidebar .full-list ul li.current-hash {
  color: var(--sidebarActiveItem);
}

.sidebar .full-list ul li.current-hash > a {
  color: var(--sidebarActiveItem);
}

.sidebar .full-list ul li.current-hash > a:before,
.sidebar .full-list > li > ul > li > a:hover:before {
  content: "\2022";
  position: absolute;
  margin-left: -15px;
  color: var(--sidebarActiveItem);
}

.sidebar .full-list ul li a {
  padding-left: 15px;
  display: block;
  width: 100%;
  height: 24px;
}

.sidebar .full-list ul li ul {
  display: none;
  margin: 9px 20px;
  margin-right: 0;
}

.sidebar .full-list ul li ul li {
  margin-right: 0;
  height: 20px;
  color: var(--sidebarAccentMain);
}

.sidebar .full-list ul li ul li a {
  border-left: 1px solid var(--sidebarInactiveItemMarker);
  padding: 0 10px;
  height: 20px;
}

.sidebar .full-list ul li ul li.current-hash > a:before {
  content: none;
}

.sidebar .full-list ul li ul li > a:hover {
  border-color: var(--sidebarLanguageAccentBar);
}

.sidebar .full-list ul li ul li.current-hash > a {
  color: var(--sidebarActiveItem);
  border-color: var(--sidebarLanguageAccentBar);
}

.sidebar .full-list ul li ul li.current-hash > a {
  color: var(--sidebarActiveItem);
  margin-left: 0;
}

.sidebar ::-webkit-scrollbar {
  width: 14px;
}

::-webkit-scrollbar-track {
  background-color: var(--sidebarBackground);
}

.sidebar ::-webkit-scrollbar-thumb {
  background-color: var(--sidebarScrollbarThumb);
  border-radius: 10px;
  border: 3px solid var(--sidebarBackground);
}

.sidebar-button {
  cursor: pointer;
  background-color: transparent;
  border: none;
  font-size: var(--sidebarFontSize);
}

.sidebar-button:hover {
  color: var(--sidebarHover);
}

.sidebar-button {
  color: var(--sidebarAccentMain);
}

.sidebar-closed .sidebar-button {
  color: var(--contrast);
}

@media screen and (max-height: 500px) {
  .sidebar {
    overflow-y: auto;
  }

  .sidebar .full-list {
    overflow: visible;
  }
}
