.content-inner section.admonition {
  border-left: solid 4px;
  color: var(--black);
  font-size: 0.9em;
  line-height: 1.4em;
  margin-bottom: 1.5em;
  margin-left: 5px;
  padding: 7px 15px;
  page-break-inside: avoid;
}

.content-inner section.admonition.warning {
  background-color: var(--warningBackground);
  border-left-color: var(--warningHeadingBackground);
}

.content-inner section.admonition.error {
  background-color: var(--errorBackground);
  border-left-color: var(--errorHeadingBackground);
}

.content-inner section.admonition.info {
  background-color: var(--infoBackground);
  border-left-color: var(--infoHeadingBackground);
}

.content-inner section.admonition.neutral {
  background-color: var(--neutralBackground);
  border-left-color: var(--neutralHeadingBackground);
}

.content-inner section.admonition.tip {
  background-color: var(--tipBackground);
  border-left-color: var(--tipHeadingBackground);
}

.content-inner section.admonition > .admonition-title {
  font-weight: bold;
  margin: 0px 10px 5px 0px;
  font-style: normal;
  font-weight: 700;
}

.content-inner section.admonition > .admonition-title.warning {
  color: var(--warningHeadingBackground);
}
.content-inner section.admonition > .admonition-title.error {
  color: var(--errorHeadingBackground);
}
.content-inner section.admonition > .admonition-title.info {
  color: var(--infoHeadingBackground);
}
.content-inner section.admonition > .admonition-title.neutral {
  color: var(--neutralHeadingBackground);
}
.content-inner section.admonition > .admonition-title.tip {
  color: var(--tipHeadingBackground);
}

.content-inner section.admonition > .admonition-title code {
  margin: 0 0.5ch;
}

.content-inner section.admonition code {
  background-color: var(--admInlineCodeBackground);
  border: 1px solid var(--admInlineCodeBorder);
  color: var(--admInlineCodeColor);
}

.content-inner section.admonition pre code {
  background-color: var(--admCodeBackground);
  border: 1px solid var(--admCodeBorder);
  color: var(--admCodeColor);
}
