.content-inner section.admonition {
  border-radius: 10px;
  border-left: 0;
}

.content-inner section.admonition.warning {
  background-color: var(--warningBackground);
}

.content-inner section.admonition.error {
  background-color: var(--errorBackground);
}

.content-inner section.admonition.info {
  background-color: var(--infoBackground);
}

.content-inner section.admonition.neutral {
  background-color: var(--neutralBackground);
}

.content-inner section.admonition.tip {
  background-color: var(--tipBackground);
}

.content-inner section.admonition > .admonition-title {
  color: var(--contrast);
  margin: 0 -1.2rem;
  padding: .7rem 1.2rem .7rem 3.3rem;
  font-weight: 700;
  font-style: normal;
}
.content-inner section.admonition > .admonition-title::before {
  color: var(--contrast);
  position: absolute;
  left: 1rem;
  font-size: 1.8rem;
  font-family: 'remixicon';
  font-style: normal;
  -webkit-font-smoothing: antialiased;
  -moz-osx-font-smoothing: grayscale;
}

.content-inner section.admonition > .admonition-title.warning {
  background-color: var(--warningHeadingBackground);
  color: var(--warningHeading);
}
.content-inner section.admonition > .admonition-title.warning::before {
  content: var(--icon-error-warning);
  color: var(--warningHeading);
}

.content-inner section.admonition > .admonition-title.error {
  background-color: var(--errorHeadingBackground);
  color: var(--errorHeading);
}
.content-inner section.admonition > .admonition-title.error::before {
  content: var(--icon-error-warning);
  color: var(--errorHeading);
}

.content-inner section.admonition > .admonition-title.info {
  background-color: var(--infoHeadingBackground);
  color: var(--infoHeading);
}
.content-inner section.admonition > .admonition-title.info::before {
  content: var(--icon-information);
  color: var(--infoHeading);
}

.content-inner section.admonition > .admonition-title.neutral {
  background-color: var(--neutralHeadingBackground);
  color: var(--neutralHeading);
}
.content-inner section.admonition > .admonition-title.neutral::before {
  content: var(--icon-double-quotes-l);
  color: var(--neutralHeading);
}

.content-inner section.admonition > .admonition-title.tip {
  background-color: var(--tipHeadingBackground);
  color: var(--tipHeading);
}
.content-inner section.admonition > .admonition-title.tip::before {
  content: var(--icon-information);
  color: var(--tipHeading);
}

.content-inner section.admonition > .admonition-title code {
  margin: 0 0.5ch;
}

.content-inner section.admonition code {
  background-color: var(--admInlineCodeBackground);
  border: 1px solid var(--admInlineCodeBorder);
  color: var(--admInlineCodeColor);
}

.content-inner section.admonition  pre code {
  background-color: var(--admCodeBackground);
  border: 1px solid var(--admCodeBorder);
  color: var(--admCodeColor);
}

.content-inner section.admonition > .admonition-title :is(a, a:visited) {
  color: inherit;
  text-decoration-color: currentColor;
}

@media screen and (max-width: 768px) {
  .content-inner section.admonition {
    margin-left: calc(-1 * var(--content-gutter));
    margin-right: calc(-1 * var(--content-gutter));
    padding-left: var(--content-gutter);
    padding-right: var(--content-gutter);
    border-radius: 0;
  }

  .content-inner section.admonition > .admonition-title {
    margin: 0 calc(-1 * var(--content-gutter));
  }
}
