@import "@fontsource/lato/300.css";
@import "@fontsource/lato/400.css";
@import "@fontsource/lato/700.css";
@import "@fontsource/inconsolata/400.css";
@import "@fontsource/inconsolata/700.css";

@import "custom-props/common.css";
@import "custom-props/theme-light.css";
@import "custom-props/theme-dark.css";

@import "normalize.css/normalize.css";

@import "icons.css";
@import "layout.css";
@import "sidebar.css";
@import "search-bar.css";
@import "focus.css";
@import "content/general.css";
@import "content/admonition.css";
@import "content/summary.css";
@import "content/code.css";
@import "content/functions.css";
@import "content/footer.css";
@import "content/bottom-actions.css";
@import "content/cheatsheet.css";
@import "search.css";
@import "modal.css";
@import "keyboard-shortcuts.css";
@import "quick-switch.css";
@import "autocomplete.css";
@import "tooltips.css";
@import "copy-button.css";
@import "settings.css";
@import "toast.css";
@import "screen-reader.css";
@import "print.css";
@import "print-cheatsheet.css";
@import "makeup.css";
@import "tabset.css";
@import "preview.css";

body:not(.dark) .content-inner img[src*="#gh-dark-mode-only"],
body.dark .content-inner img[src*="#gh-light-mode-only"] {
  display: none;
}
