@import 'custom-props/common.css';
@import 'custom-props/theme-light.css';

@import 'content/epub-admonition.css';
@import 'content/code.css';
@import 'content/functions.css';
@import 'screen-reader.css';
@import 'makeup.css';

body {
  display: block;
  font-size: 1em;
  line-height: 1.2;
  padding-left: 0;
  padding-right: 0;
  margin: 0 5pt;
}

nav > ol {
  list-style-type: square;
}

nav > ol ol {
  list-style-type: disc;
}

.title-container {
  text-align: center;
}

img[src*="#gh-dark-mode-only"] {
  display: none;
}
